
Agent = Core.class(Bitmap)

function Agent:init(texture) -- this only needs to be called once
  self:reset()  
end

function Agent:reset() -- we call this to actually initialize the agent
  local x, y = 0, 0
  
  self.position = {x = x, y = y}
  self.velocity = {x = 0, y = 0}

  self.speed = 0.3 -- how fast this agent can move (in pixels per frame)

  self.state = "nothing" -- what the agent is currently doing
  
  self.direction = "right" -- which direction the agent is travelling
  self.path = {}
  self.pathPos = 1
  self.destination = {}

--  if BUMP:hasItem(self) then
--    BUMP:update(self, x, y) -- just update if it already exists in bump
--  else
--    BUMP:add(self, x, y, 16, 16) -- otherwise add it
--  end
end

function Agent:setState(state) -- set what this agent will do from now on
  self.state = state
end

function Agent:setPath(path) -- set the path this agent will follow
  self.path = path
  self.pathPos = 1
  self.destination = path[1]
  self:chooseDirection() -- choose movement direction
end

function Agent:advancePath() -- set agent to move towards next waypoint in current path
  local path = self.path
  local pathPos = self.pathPos + 1
  
  if pathPos > #path then
    
    -- at this point the agent has completed the path so you could choose 
    -- a new state or choose and generate a new path. for this example we 
    -- are just resetting the path position to the start so the agent will 
    -- walk about in a loop.
    
    pathPos = 1
  end
  
  self.destination = path[pathPos] -- set new destination
  self.pathPos = pathPos
  self:chooseDirection() -- choose movement direction
end

function Agent:chooseDirection() -- choose movement direction to next waypoint
  local p = self.position
  local v = self.velocity
  local d = self.destination
  
  local x, y = p.x, p.y -- current world position
  local vx, vy = 0, 0 -- initial velocities
  local dx, dy = d.x, d.y -- destination position
  
  local speed = self.speed -- how fast this agent can move
  
  if dx ~= x then -- if x is different then we will move on x axis (left/right)
    
    if dx < x then -- if destination is to left we will move left
      self.direction = "left"
      vx = -speed
    else -- otherwise we will move right
      self.direction = "right"
      vx = speed
    end

  else -- otherwise we will move on y axis (up/down)
    
    if dy < y then -- if destination is to above we will move up
      self.direction = "up"
      vy = -speed
    else -- otherwise we will move down
      self.direction = "down"
      vy = speed
    end
  
  end
  
  v.x, v.y = vx, vy -- set velocities
end

function Agent:update(dt)
  local p = self.position
  local v = self.velocity
  local d = self.destination
  
  local x, y = p.x + v.x, p.y + v.y -- move
  local dx, dy = d.x, d.y -- we check against these
  
  local state = self.state
  if state == "patrol" then
    local advance = false
    
    local path = self.path
    local dir = self.direction
    if dir == "left" then
      if x <= dx then
        x = dx
        advance = true
      end
    elseif dir == "right" then
      if x >= dx then
        x = dx
        advance = true
      end
    elseif dir == "up" then
      if y <= dy then
        y = dy
        advance = true
      end
    else -- must be down then
      if y >= dy then
        y = dy
        advance = true
      end
    end
  
    p.x, p.y = x, y -- update positions
    self:setPosition(x, y)
--    BUMP:update(self, x, y) -- agents only need to be updated.. not moved

    if advance then -- advance path if required
      self:advancePath()
    end
  
  elseif state == "" then
    -- some other state
  else
    -- no state
  end
  
  -- process stuff here that happens regardless of state, things like
  -- checking if the agent can see the player, animating the agent, etc
  
end

